<?php
/*
    Copyright (c) 2002-2007 Netlor
    Copyright (c) 2007-2008 Ovensia
    Contributors hold Copyright (c) to their code submissions.

    This file is part of Ploopi.

    Ploopi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ploopi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ploopi; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Outil de diagnostic pour dtecter d'ventuels problmes de configuration/installation
 *
 * @package system
 * @subpackage system_tools
 * @copyright Netlor, Ovensia
 * @license GNU General Public License (GPL)
 * @author Stphane Escaich
 */

/**
 * Ouverture du bloc
 */
echo $skin->open_simplebloc(_SYSTEM_LABEL_DIAGNOSTIC);

$columns = array();
$values = array();

$columns['left']['function']    = array('label' => _SYSTEM_LABEL_FUNCTION, 'width' => '200', 'options' => array('sort' => true));
$columns['left']['desc']        = array('label' => _SYSTEM_LABEL_DESCRIPTION, 'width' => '250', 'options' => array('sort' => true));
$columns['right']['result']     = array('label' => _SYSTEM_LABEL_RESULT, 'width' => '80', 'options' => array('sort' => true));
$columns['auto']['comment']     = array('label' => _SYSTEM_LABEL_COMMENTARY, 'options' => array('sort' => true));

$c = 0;

$array_path = array();
$array_path['data'] = _PLOOPI_PATHDATA;
$array_path['modules'] = realpath('.')._PLOOPI_SEP.'modules';

/* TEST 1 - Ecriture dans les dossiers DATA et MODULES */

foreach($array_path as $key => $path)
{
    $comment = 'Le dossier est accessible en criture.';
    $testok = true;

    switch($key)
    {
        case 'data':
            $desc = "Le dossier  {$key}  est utilis par PLOOPI pour stocker les fichiers envoys par l'utilisateur ou gnrs  partir des modules. Ce dossier doit tre accessible en criture sinon certains modules peuvent ne pas fonctionner correctement.";
        break;

        case 'modules':
            $desc = "Le dossier  {$key}  est utilis par PLOOPI lors de l'installation d'un module. Ce dossier doit tre accessible en criture durant la phase d'installation des modules. L'accs en criture peut tre dsactiv le reste du temps.";
        break;
    }

    if (!file_exists($path))
    {
        $comment = "Le dossier  $path  n'existe pas";
        $testok = false;
    }
    else
    {
        $path_stat = stat($path);
        $path_pwuid = posix_getpwuid($path_stat['uid']);
        $path_grgid = posix_getgrgid($path_stat['gid']);
        $path_username = $path_pwuid['name'];
        $path_goupname = $path_grgid['name'];

        $path_perm = substr(decoct($path_stat['mode']),-4);
        $path_owner = $path_stat['uid'];

        $apache_pwuid = posix_getpwuid(posix_getuid());
        $apache_grgid = posix_getgrgid(posix_getgid());
        $apache_username = $apache_pwuid['name'];
        $apache_goupname = $apache_grgid['name'];

        if (!is_writable($path))
        {

            $comment = "Le dossier  {$path}  n'est pas accessible en criture.\nIl appartient actuellement  l'utilisateur {$path_username} du groupe {$path_goupname}.\nPour rappel, l'utilisateur Apache ({$apache_username}) du groupe {$apache_goupname} doit pouvoir crire dans ce dossier.";
            $testok = false;
        }
    }

    switch($key)
    {
        case 'data':
            $bullet = ($testok) ? 'green' : 'red';
        break;

        case 'modules':
            $bullet = ($testok) ? 'green' : 'orange';
            $comment .= ($testok) ? '' : "\nIl n'est pas possible d'installer des modules dans cette configuration.";
        break;
    }

    $values[$c]['values']['function']   = array('label' => ploopi_htmlentities("Accs en criture au dossier  {$key} "));
    $values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities($desc)), 'style' => '');
    $values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
    $values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
    $c++;

}

/* TEST 2 - config PHP */

$testpear = (file_exists(_PLOOPI_PEARPATH.'/PEAR.php'));

$comment = ($testpear) ? 'PEAR est correctement configur.' : 'Vous devez modifier le chemin vers PEAR.';
$bullet = ($testpear) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => "PEAR");
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br("Le framework PEAR est indispensable au bon fonctionnement de Ploopi."), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

if ($testpear)
{
    $testpearinfo = (file_exists(_PLOOPI_PEARPATH.'/PEAR/Info.php'));

    $comment = ($testpearinfo) ? 'PEAR_info est install.' : "Vous devez installer la classe PEAR PEAR_info.\nPour l'installer, faites &laquo; pear install --alldeps PEAR_info &raquo;.";
    $bullet = ($testpearinfo) ? 'green' : 'red';

    $values[$c]['values']['function']   = array('label' => "PEAR - PEAR_info");
    $values[$c]['values']['desc']       = array('label' => ploopi_nl2br("Informations sur PEAR.\nAffichage des dpendances."), 'style' => '');
    $values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
    $values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
    $c++;

    if ($testpearinfo)
    {
        ploopi_unset_error_handler();
        require_once 'PEAR/Info.php';
        $packPEAR = new PEAR_Info(_PLOOPI_PEARPATH); // Class PEAR_Info for test if modules pear are installed
        ploopi_set_error_handler();

        $arrPearClasses = array(
            'Cache_Lite' => array('Gestion du cache.', '1.7.15', null),
            'HTTP_Request2' => array('Gestion des connexions sortantes.', '2.1.1', null),
            'Text_Highlighter' => array('Gestion de la colorisation syntaxique.', '0.7.3', null),
            'Net_UserAgent_Detect' => array('Dtection du UserAgent.', '2.5.2', null),
            'Horde_Text_Diff' => array('Gestion du diffrentiel.', '2', 'pear.horde.org'),
            'XML_Serializer' => array('Gnration XML', '0.20.2', '')
        );


        foreach($arrPearClasses as $strPearClass => $desc)
        {
            ploopi_unset_error_handler();
            $testok = $packPEAR->packageInstalled($strPearClass, $desc[1], $desc[2]);
            ploopi_set_error_handler();

            $comment = ($testok) ? "{$strPearClass} est install." : "Vous devez installer la classe PEAR {$strPearClass}.\nPour l'installer, faites &laquo; pear install --alldeps {$strPearClass} &raquo;.";
            $bullet = ($testok) ? 'green' : 'orange';

            $values[$c]['values']['function']   = array('label' => "PEAR - {$strPearClass}");
            $values[$c]['values']['desc']       = array('label' => ploopi_nl2br($desc[0]));
            $values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment));
            $values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />");
            $c++;
        }
    }
}

$testok = !get_magic_quotes_gpc();

$comment = ($testok) ? 'La directive est correctement configure.' : 'Vous devriez modifier la valeur de cette directive.';
$bullet = ($testok) ? 'green' : 'orange';

$values[$c]['values']['function']   = array('label' => ploopi_htmlentities("PHP - magic_quote_gpc"));
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities("La directive php 'magic_quote_gpc' permet d'ajouter automatiquement des  '  dans le contenu des superglobales \$_GET, \$_POST, \$_COOKIE. Il est recommand de dsactiver cette fonctionnalit.")), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$testok = !ini_get('register_globals');

$comment = ($testok) ? 'La directive est correctement configure.' : 'Vous devriez modifier la valeur de cette directive.';
$bullet = ($testok) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => ploopi_htmlentities("PHP - register_globals"));
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities("La directive php 'register_globals' permet d'affecter automatiquement les paramtres en variables globales. Il est fortement recommand de dsactiver cette fonctionnalit.")), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$testok = !ini_get('display_errors');

$comment = ($testok) ? 'La directive est correctement configure.' : 'Vous devriez modifier la valeur de cette directive.';
$bullet = ($testok) ? 'green' : 'orange';

$values[$c]['values']['function']   = array('label' => ploopi_htmlentities("PHP - display_errors"));
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities("La directive php 'display_errors' permet d'afficher les erreurs d'excution. Il est fortement recommand de dsactiver cette fonctionnalit, notamment pour un site en production.")), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$mem = intval(ini_get('memory_limit'));
$testok = $mem >= 128;

$comment = ($testok) ? 'La directive est correctement configure.' : 'Vous devriez modifier la valeur de cette directive.';
$bullet = ($testok) ? 'green' : 'orange';

$comment .= " (Valeur actuelle : {$mem} Mio)";

$values[$c]['values']['function']   = array('label' => ploopi_htmlentities("PHP - memory_limit"));
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities("La directive php 'memory_limit' est utile pour les scripts qui consomment beaucoup de mmoire (comme l'indexation des documents ou le moteur de recherche).")), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$ploopi_maxfilesize = sprintf('%.02f', _PLOOPI_MAXFILESIZE/1024);
$upload_max_filesize =  intval(ini_get('upload_max_filesize')*1024);
$post_max_size = intval(ini_get('post_max_size')*1024);

// Test la taille max d'upload de fichier
$fmax = (_PLOOPI_USE_CGIUPLOAD) ? $ploopi_maxfilesize : min($ploopi_maxfilesize, $upload_max_filesize, $post_max_size);

$testok = 1;
if ($fmax < 2048) $testok = 3; // 2 Mo
elseif ($fmax < 5120) $testok = 2; // 5 Mo

$fmax = sprintf("%.02f", $fmax/1024);

switch($testok)
{
    case 1:
        $bullet = 'green';
        $comment = 'Vous pouvez uploader des fichiers d\'un poids suprieur  5 Mio (max : '.$fmax.' Mio).';
    break;

    case 2:
        $bullet = 'orange';
        $comment = 'Vous pouvez uploader des fichiers d\'un poids maximum infrieur  5 Mio.';
    break;

    case 3:
        $bullet = 'red';
        $comment = 'Vous pouvez uploader des fichiers d\'un poids maximum infrieur 2 Mio.';
    break;
}

if (_PLOOPI_USE_CGIUPLOAD) $comment .= "\nLe mode CGIUPLOAD est activ. Attention car il ncessite une configuration particulire d'Apache";

if ($testok>1) $comment .= "\nSi vous voulez modifier cette limite vous pouvez modifier les paramtres suivants:\n- upload_max_filesize (PHP / php.ini) : {$upload_max_filesize} kio\n- post_max_size (PHP / php.ini) : {$post_max_size} kio\n- _PLOOPI_MAXFILESIZE (PLOOPI / config.php) : {$ploopi_maxfilesize} kio";

$values[$c]['values']['function']   = array('label' => ploopi_htmlentities("Capacit d'upload"));
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br(ploopi_htmlentities("La capacit d'upload permet de dterminer le poids maximum d'un fichier pouvant tre accept par le serveur. Si le poids est trop petit, vous risquez de ne pas pouvoir accepter certains documents.")), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br(ploopi_htmlentities($comment)), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$testok = extension_loaded('gd');

$comment = ($testok) ? 'L\'extension &laquo; gd &raquo; est active.' : 'Vous devriez activer l\'extension &laquo; gd &raquo;.';
$bullet = ($testok) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => "PHP Extension - gd");
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br("L'extension &laquo; gd &raquo; permet de traiter les images."), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$testok = extension_loaded('mcrypt');

$comment = ($testok) ? 'L\'extension &laquo; mcrypt &raquo; est active.' : 'Vous devriez activer l\'extension &laquo; mcrypt &raquo;.';
$bullet = ($testok) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => "PHP Extension - mcrypt");
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br("L'extension &laquo; mcrypt &raquo; permet de gnrer les url encodes."), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$testok = extension_loaded('stem');

$comment = ($testok) ? 'L\'extension &laquo; stem &raquo; est active.' : 'Vous devriez activer l\'extension &laquo; stem &raquo;.';
$bullet = ($testok) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => "PHP Extension - stem");
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br("L'extension &laquo; stem &raquo; permet d'indexer certains contenus."), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

/* TEST 3 - Connectivit internet */
$testurl = 'http://www.ploopi.org';
if ($testpear)
{
    $comment = 'Connexion internet ouverte.';
    $testok = true;

    require_once 'HTTP/Request2.php';

    $objRequest = new HTTP_Request2($testurl);

    if (_PLOOPI_INTERNETPROXY_HOST != '')
    {
        $arrConfig['proxy_host'] = _PLOOPI_INTERNETPROXY_HOST;
        $arrConfig['proxy_port'] = _PLOOPI_INTERNETPROXY_PORT;
        $arrConfig['proxy_user'] = _PLOOPI_INTERNETPROXY_USER;
        $arrConfig['proxy_password'] = _PLOOPI_INTERNETPROXY_PASS;
        $arrConfig['proxy_auth_scheme'] = HTTP_Request2::AUTH_BASIC;
        $objRequest->setConfig($arrConfig);
    }


    $objResponse = $objRequest->send();

    $strStatus = $objResponse->getStatus();

    $res = $strStatus == '200' || $objResponse->getStatus() == '';
}
else $res = false;

if (!$res)
{
    $comment = "Problme de connexion internet.\nPLOOPI n'a pas pu se connecter sur <a title=\"{$testurl}\" href=\"{$testurl}\">{$testurl}</a>.";
    if (_PLOOPI_INTERNETPROXY_HOST != '') $comment .= "\nen utilisant les paramtres Proxy suivants :\nproxy_host: "._PLOOPI_INTERNETPROXY_HOST.", proxy_port: "._PLOOPI_INTERNETPROXY_PORT.", proxy_user: "._PLOOPI_INTERNETPROXY_USER.", proxy_pass: "._PLOOPI_INTERNETPROXY_PASS;

    $testok = false;
}

$bullet = ($testok) ? 'green' : 'red';

$values[$c]['values']['function']   = array('label' => "Connexion internet");
$values[$c]['values']['desc']       = array('label' => ploopi_nl2br("Certains modules de PLOOPI ont besoin de se connecter  internet. Ce test vous indique si le serveur arrive  ouvrir une connexion internet."), 'style' => '');
$values[$c]['values']['comment']    = array('label' => ploopi_nl2br($comment), 'style' => '');
$values[$c]['values']['result']     = array('label' => "<img src=\"{$_SESSION['ploopi']['template_path']}/img/system/p_{$bullet}.png\" />", 'style' => '');
$c++;

$skin->display_array($columns, $values, 'array_diagnostic', array('sortable' => true, 'orderby_default' => 'function'));
?>

<p class="ploopi_va" style="padding:2px;">
<img src="<?php echo $_SESSION['ploopi']['template_path']; ?>/img/system/p_red.png">
<span>Une puce rouge indique un problme potentiellement bloquant</span>
</p>
<p class="ploopi_va" style="padding:2px;">
<img src="<?php echo $_SESSION['ploopi']['template_path']; ?>/img/system/p_orange.png">
<span>Une puce orange indique qu'une fonctionnalit ou une configuration peut poser problme</span>
</p>
<p class="ploopi_va" style="padding:2px;">
<img src="<?php echo $_SESSION['ploopi']['template_path']; ?>/img/system/p_green.png">
<span>Une puce verte indique que tout va bien !</span>
</p>

<?php echo $skin->close_simplebloc(); ?>
